package com.tzld.longarticle.recommend.server.service.exterior.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.mapper.crawler.GhDetailMapper;
import com.tzld.longarticle.recommend.server.repository.model.GhDetail;
import com.tzld.longarticle.recommend.server.repository.model.GhDetailExample;
import com.tzld.longarticle.recommend.server.service.exterior.GhDetailService;
import com.tzld.longarticle.recommend.server.util.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import java.util.List;

@Slf4j
@Service
public class GhDetailServiceImpl implements GhDetailService {

    @Autowired
    private GhDetailMapper ghDetailMapper;

    @Override
    public CommonResponse<Page<GhDetail>> getGhDetailList(Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        if (pageSize > 100) {
            pageSize = 100;
        }
        Page<GhDetail> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        GhDetailExample example = new GhDetailExample();
        example.setPage(page);
        long total = ghDetailMapper.countByExample(example);
        int totalSize = (int) (total % pageSize == 0 ? (total / pageSize) : (total / pageSize + 1));
        page.setTotalSize(totalSize);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        page.setObjs(ghDetails);
        return CommonResponse.success(page);
    }

    @Override
    public CommonResponse<Void> addGhDetail(GhDetail ghDetail) {
        ghDetailMapper.insert(ghDetail);
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> updateDetail(GhDetail ghDetail) {
        ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> refreshGhDetail() {
        GhDetailExample example = new GhDetailExample();
        example.createCriteria().andTypeEqualTo(1);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(ghDetails)) {
            return CommonResponse.success();
        }
        for (GhDetail ghDetail : ghDetails) {
            getGhDetail(ghDetail);
            log.info("getGhDetail = {}", ghDetail);
            ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
        }
        return CommonResponse.success();
    }

    public void getGhDetail(GhDetail ghDetail) {
        String baseUrl = "http://aigc-api.cybertogether.net";
        WebClient webClient = WebClient.builder()
                .baseUrl(baseUrl)
                .build();
        Mono<String> mono = webClient.get()
                .uri(String.format("/aigc/publish/account/algorithm/wechat/list?gh_id=%s", ghDetail.getGhId()))
                .retrieve().bodyToMono(String.class);
        String block = mono.block();
        JSONArray jsonArray = JSONArray.parseArray(block);
        if (jsonArray != null && !jsonArray.isEmpty()) {
            JSONObject jsonObject = jsonArray.getJSONObject(0);
            ghDetail.setGhName(jsonObject.getString("name"));
        }
    }
}
